@@ -898,6 +898,12 @@ PHPAPI ZEND_COLD void php_print_info(int flag)
 			efree(descr);
 		}
 
+#ifdef ZEND_MAX_EXECUTION_TIMERS
+		php_info_print_table_row(2, "Zend Max Execution Timers", "enabled" );
+#else
+		php_info_print_table_row(2, "Zend Max Execution Timers", "disabled" );
+#endif
+
 #ifdef HAVE_IPV6
 		php_info_print_table_row(2, "IPv6 Support", "enabled" );
 #else
